/*
 * \file FortuneClient.cpp
 *
 *  \date Oct 16, 2013
 *  \author Alexandre Tuleu
 */

#include <biorob-cpp/net/TcpSocket.h>

#include <signal.h>
#include <iostream>
#include <sstream>
#include <cstdlib>


class Application {
public :
	typedef std::tr1::shared_ptr<Application> Ptr;
	Application();
	~Application();
	void Run();
};


static Application::Ptr app;

void SigHandler(int);

void SigHandler(int sig) {
	if(sig == SIGPIPE) {
		std::cerr << "Got a broken PIPE." << std::endl;
	}
	app = Application::Ptr();
	exit(sig == SIGPIPE ? 42 : 0);
}


int main() {
	signal(SIGINT,&SigHandler);
	signal(SIGPIPE,&SigHandler);

	int ret = 0;
	try {
		app = Application::Ptr(new Application());
		app->Run();
	} catch (const std::exception & e) {
		std::cerr << "Got error : " << e.what() << std::endl;
		ret = 1;
	}

	app = Application::Ptr();
	return ret;
}


Application::Application(){}

Application::~Application(){
}



void Application::Run() {
	std::cout << "Connecting to localhost:12345" << std::endl;
	net::TcpSocket s("localhost",12345);
	char buffer[13];
	s.ReadAll((uint8_t*)buffer,13);
	if(buffer[12] != '\0') {
		std::ostringstream os;
		for(unsigned int i = 0 ; i < 13; ++i) {
			os << buffer[i];
		}
		throw std::runtime_error("Did not get a NULL terminated string from server, but '" + os.str() + "'");
	}

	std::cout << "Got message : '" << buffer << "'." << std::endl;
}

