/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file PluginUTest.cpp
 *
 *  \date Dec 3, 2012
 *  \author Alexandre Tuleu
 */

#include "PluginUTestConfig.h"

#include "PluginUTest.h"

void PluginUTest::SetUp(){
	d_loader.SetHint((PluginLoader<DummyPlugin>::LoadingHint)
	                 (PluginLoader<DummyPlugin>::ResolveAllSymbols |
	                  PluginLoader<DummyPlugin>::IsCMakeModuleObject ));
	d_pluginDir= CMAKE_CURRENT_BINARY_DIR;
	d_pluginDir += "/";

	d_pluginOutputByName["plugin-a"] = "I am plugin A.\n";
	d_pluginOutputByName["plugin-b"] = "I am plugin B.\n";
}

void PluginUTest::TearDown(){
}


TEST_F(PluginUTest,LoadPluginSequentially){
	for(PluginOutputByName::const_iterator pit = d_pluginOutputByName.begin();
	    pit != d_pluginOutputByName.end();
	    ++pit){
		d_loader.SetFilename( d_pluginDir + pit->first);
		std::ostringstream out;
		try{
			PluginLoader<DummyPlugin>::PluginPtr p = d_loader.LoadPlugin();
			p->Print(out);
		} catch (const std::exception & e){
			FAIL() << "Got exception : " << e.what();
		}
		EXPECT_EQ(pit->second,out.str());
	}
}

TEST_F(PluginUTest,LoadOnlyFirstPlugin){
	d_loader.AppendVariant(d_pluginDir + "plugin-do-no-exists");
	d_loader.AppendVariant(d_pluginDir + "plugin-b");
	d_loader.AppendVariant(d_pluginDir + "plugin-a");

	std::ostringstream out;
	try{
		PluginLoader<DummyPlugin>::PluginPtr p = d_loader.LoadPlugin();
		p->Print(out);
	} catch (const std::exception & e) {
		FAIL() << "Got exception : " << e.what();
	}
	EXPECT_EQ(d_pluginOutputByName["plugin-b"],out.str());
}

TEST_F(PluginUTest,GetNiceExceptionOnLoadingError){
	d_loader.SetFilename("do-no-exists");
	EXPECT_THROW({
		PluginLoader<DummyPlugin>::PluginPtr p = d_loader.LoadPlugin();
	}, std::runtime_error );

}

int main(int argc, char ** argv){
	::testing::InitGoogleTest(&argc,argv);
	return RUN_ALL_TESTS();
}
