/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Copyright (c) 2012  Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Variable.h
 *
 *  \date Mar 27, 2012
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CLI_VARIABLE_H_
#define BIOROB_CLI_VARIABLE_H_


#include <string>
#include <sstream>
#include <tr1/memory>


namespace cli {
/**
 *
 */
class VariableBase {
public:
	typedef std::tr1::shared_ptr<VariableBase> Ptr;
	VariableBase(const std::string & name);
	virtual ~VariableBase();

	const std::string & Name() const;

	virtual const std::string & toString() const = 0;
	virtual bool SetFromString(const std::string & value) = 0;
	virtual void PostSetValueHook();
	virtual void PreGetValueHook() const;
private :
	const std::string d_name;
};

template <typename T>
class Variable : public VariableBase{
public :
	Variable(const std::string & name, const T & value);
	virtual ~Variable();
	const T & Value() const;
	void SetValue(const T & value);
	const std::string & toString() const;
	bool SetFromString(const std::string & value);

protected :

	T & ProtectedValue();

private :
	void SetStringFromActualValue();

	T d_value;
	std::string d_valueString;
};


/******************************************************************************
 * Inline Implementation
 *****************************************************************************/

template <typename T>
inline Variable<T>::Variable(const std::string & name, const T & value)
	: VariableBase(name)
	, d_value(value)
	, d_valueString(){
	SetStringFromActualValue();
}



template <typename T>
inline Variable<T>::~Variable(){}



template <typename T>
inline const T & Variable<T>::Value() const{
	PreGetValueHook();
	return d_value;
}



template <typename T>
inline void Variable<T>::SetValue(const T & value){
	d_value = value;
	SetStringFromActualValue();
	PostSetValueHook();
}



template <typename T>
inline
const std::string & Variable<T>::toString() const{
	return d_valueString;
}


template <typename T>
inline bool Variable<T>::SetFromString(const std::string & value){
	std::istringstream iss(value);
	T tmp;
	iss >> tmp;
	if(iss.fail()){
		return false;
	}
	SetValue(tmp);
	return true;
}



template <typename T>
inline void Variable<T>::SetStringFromActualValue(){
	std::ostringstream os;
	os << d_value << std::flush;
	d_valueString = os.str();
}





template<>
class Variable<std::string> : public VariableBase{
public :
	inline Variable(const std::string & name, const std::string & value)
		: VariableBase(name)
		, d_value(value){}

	inline const std::string & Value() const{
		return d_value;
	}

	inline void SetValue(const std::string & value){
		d_value = value;
	}

	inline const std::string & toString() const{
		return d_value;
	}

	bool SetFromString(const std::string & value){
		d_value = value;
		return true;
	}
private :
	std::string d_value;
};


template <typename T>
inline T & Variable<T>::ProtectedValue(){
	return d_value;
}

} //namespace cli
#endif // BIOROB_CLI_VARIABLE_H_
