/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Singleton.h
 *
 *  \date Dec 20, 2010
 *  \author Alexandre Tuleu
 */
#include <set>

#include "common.h"


#ifndef BIOROB_CPP_BASE_SINGLETON_H_
#define BIOROB_CPP_BASE_SINGLETON_H_

template <typename T>

class LazySingleton{
DISABLE_COPY(LazySingleton)
public :
	virtual ~LazySingleton();
	static T & Instance();
protected :
	LazySingleton();
};


template <typename T>
class Singleton {
DISABLE_COPY(Singleton)
public :
	virtual ~Singleton();
	static T & Instance();
protected :
	Singleton();
	virtual void Init();
private :
	static T s_instance;
};


template <typename T>
class ManualSingleton {
	DISABLE_COPY(ManualSingleton)
public :
	virtual ~ManualSingleton();
	static void Initialize(T * t);
	static T & Instance();
protected :
	ManualSingleton();
private :
	static T * s_instance;
};

#include "Singleton.ih"

#endif // BIOROB_CPP_BASE_SINGLETON_H_
