/*
 * \file Hostname.cpp
 *
 *  \date Oct 21, 2013
 *  \author Alexandre Tuleu
 */

#include "Hostname.h"

#include <unistd.h>

#include <biorob-cpp/os/SystemError.h>

namespace net {

std::string Hostname::Get() {
	size_t size = 255;
	char * buffer = new char[size];
	while(true) {
		int error = gethostname(buffer,size);
		if(error == 0) {
			break;
		}
		if(errno == ENAMETOOLONG) {
			size *= 2;
			delete []buffer;
			buffer = new char[size];
			continue;
		}

		throw os::SystemError("gethostname() returned with error",errno);
	}

	std::string res(buffer);

	delete []buffer;

	return res;
}

void Hostname::Set(const std::string & hostname) {
	p_call(sethostname,hostname.c_str(),hostname.size());
}

} /* namespace net */
