/*
 * \file Publisher.cpp
 *
 *  \date Oct 18, 2013
 *  \author Alexandre Tuleu
 */

#include "Publisher.h"

#include <stdexcept>

namespace net {
namespace dnssd {

#define THROW_IF_STARTED() do{ \
	if(this->d_started) { \
		throw std::logic_error("Configuration while in main loop is not supported."); \
	} \
}while(0)

const std::string Publisher::ANY_DOMAIN;
const std::string Publisher::ANY_HOST;
const Publisher::ListOfTxtDataFields Publisher::NO_TXT_DATA_FIELD;
const Publisher::ListOfTxtDataFields Publisher::NO_SUBTYPE;
Publisher::Publisher(const std::string & name)
	: d_name(name)
	, d_primpl(PublisherPrimpl::Create(*this))
	, d_started(false){
}

Publisher::~Publisher(){
}

void Publisher::SetName(const std::string& name){
	THROW_IF_STARTED();
	d_name = name;
}

const std::string& Publisher::Name() const{
	return d_name;
}

void Publisher::AddService(Interface interface ,
                           Protocol portocol ,
                           const std::string& type , unsigned int port ,
                           const std::string& domain , const std::string& host ,
                           const ListOfTxtDataFields& txtData ,
                           const ListOfSubtype& subtypes){
	THROW_IF_STARTED();
	ServiceEntry entry;
	entry.interface = interface;
	entry.protocol = portocol;
	entry.type = type;
	entry.port = port;
	entry.domain = domain;
	entry.host = host;
	entry.txtData = txtData;
	entry.subtypes = subtypes;

	d_services.push_back(entry);

}

void Publisher::EnterMainLoop(){
	if(d_started) {
		throw std::logic_error("Could not start twice !");
	}
	d_started = true;
	//will block the current thread.
	d_primpl->EnterMainLoop();
}


PublisherPrimpl::~PublisherPrimpl(){}

PublisherPrimpl::PublisherPrimpl(Publisher& publisher)
	: d_publisher(publisher){}

Publisher& PublisherPrimpl::GetPublisher() const{
	return d_publisher;
}

const PublisherPrimpl::ListOfServices& PublisherPrimpl::Services() const{
	return d_publisher.d_services;
}

} /* namespace dnsd */
} /* namespace net */
