/*
 * \file Publisher.h
 *
 *  \date Oct 18, 2013
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CPP_NET_DNSSD_PUBLISHER_H_
#define BIOROB_CPP_NET_DNSSD_PUBLISHER_H_

#include "Common.h"

#include <string>
#include <vector>
#include <tr1/memory>

namespace net {
namespace dnssd {


// forward declare a private implementation class
class PublisherPrimpl;

class Publisher {
public:
	typedef std::vector<std::string> ListOfTxtDataFields;
	typedef std::vector<std::string> ListOfSubtype;
	const static std::string ANY_DOMAIN;
	const static std::string ANY_HOST;
	const static ListOfTxtDataFields NO_TXT_DATA_FIELD;
	const static ListOfTxtDataFields NO_SUBTYPE;

	Publisher(const std::string & name);
	virtual ~Publisher();

	void SetName(const std::string & name);
	const std::string & Name() const;

	void AddService(Interface interface,
	                Protocol portocol,
	                const std::string & type,
	                unsigned int port,
	                const std::string & domain = ANY_DOMAIN,
	                const std::string & host = ANY_HOST,
	                const ListOfTxtDataFields & txtData = NO_TXT_DATA_FIELD,
	                const ListOfSubtype & subtype = NO_SUBTYPE);

	void EnterMainLoop();

private :
	friend class PublisherPrimpl;
	typedef std::tr1::shared_ptr<PublisherPrimpl> PrimplPtr;

	struct ServiceEntry {
		Interface           interface;
		Protocol            protocol;
		std::string         type;
		unsigned int        port;
		std::string         domain;
		std::string         host;
		ListOfTxtDataFields txtData;
		ListOfSubtype       subtypes;
	};

	typedef std::vector<ServiceEntry> ListOfServices;



	std::string     d_name;
	ListOfServices  d_services;

	PrimplPtr              d_primpl;
	bool                   d_started;
};

/**
 * \private
 * Private Interface subject to change
 */
class PublisherPrimpl {
public :
	typedef std::tr1::shared_ptr<PublisherPrimpl> Ptr;
	static PublisherPrimpl::Ptr Create(Publisher & publisher);

	virtual ~PublisherPrimpl();

	virtual void EnterMainLoop() = 0;

protected :
	typedef Publisher::ListOfServices ListOfServices;
	typedef Publisher::ServiceEntry ServiceEntry;

	PublisherPrimpl(Publisher & publisher);
	Publisher & GetPublisher() const;
	const ListOfServices & Services() const;

private :
	Publisher & d_publisher;
};

} /* namespace dnssd */
} /* namespace net */

#endif /* BIOROB_CPP_PUBLISHER_H_ */
