/*
 * \file CliArgumentParserUTest.h
 *
 *  \date Oct 10, 2013
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CPP_CLIARGUMENTPARSERUTEST_H_
#define BIOROB_CPP_CLIARGUMENTPARSERUTEST_H_


#include <gtest/gtest.h>
#include <list>

#include "CliArgParser.h"

class CLIArgsSplitter {
public :
	CLIArgsSplitter(const std::string & cli);
	~CLIArgsSplitter();
	int Argc();
	char ** Argv();



private :
	int d_argc;
	char ** d_argv;

	void Init(const std::string & cli);
};


class CliArgParserUTest : public ::testing::Test {
protected :
	void SetUp();
	void TearDown();

	template <typename T> void ParseOneParameter(T valueInitial, const std::string valueToParse);

	static std::string s_fooCommandDescription;
};

template <typename T>
inline void CliArgParserUTest::ParseOneParameter(T valueInitial,
                                                      const std::string valueToParse) {
	T value = valueInitial;
	T valueExpected;
	std::istringstream iss(valueToParse);
	iss >> valueExpected;

	EXPECT_NE(valueInitial,valueExpected);

	CLIArgsSplitter args("foo-command --option-foo " + valueToParse);

	options::CliArgParser p(s_fooCommandDescription);

	p.AddParameter<T>("option-foo",value,"a chombier parameter",true);

	EXPECT_EQ(valueInitial,value);

	EXPECT_NO_THROW(p.Parse(args.Argc(), 
	                        args.Argv(),
	                        options::CliArgParser::THROW_ON_ERROR));

	EXPECT_EQ(valueExpected,value);

}


#endif /* BIOROB_CPP_CLIARGUMENTPARSERUTEST_H_ */
