/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file OptionUTest.cpp
 *
 *  \date Dec 1, 2011
 *  \author Alexandre Tuleu
 */

#include "Option.h"
#include "OptionUTest.h"

OptionUTest::~OptionUTest(){}

TEST_F(OptionUTest, ParsingIsOK){
	options::Option<double>      myDouble("my-double",1.0);
	options::Option<int>         myInt("my-int",42);
	options::Option<bool>        myBool("my-bool",true);
	options::Option<std::string> myString("my-string","");
	options::OptionList<double>  myList("my-list");

	EXPECT_FALSE(myDouble.Parse(""));
	EXPECT_FALSE(myDouble.Parse(" "));

	EXPECT_TRUE(myDouble.Parse("1.0"));
	EXPECT_DOUBLE_EQ(1.0,myDouble.Value());

	EXPECT_TRUE(myDouble.Parse("1.0   "));
	EXPECT_DOUBLE_EQ(1.0,myDouble.Value());

	EXPECT_TRUE(myInt.Parse("69"));
	EXPECT_EQ(69,myInt.Value());

	EXPECT_TRUE(myBool.Parse("thisisanything "));
	EXPECT_TRUE(myBool.Value());

	EXPECT_TRUE(myBool.Parse("off "));
	EXPECT_FALSE(myBool.Value());

	EXPECT_TRUE(myBool.Parse("  "));
	EXPECT_FALSE(myBool.Value());

	EXPECT_TRUE(myBool.Parse("false"));
	EXPECT_FALSE(myBool.Value());

	EXPECT_TRUE(myBool.Parse("no"));
	EXPECT_FALSE(myBool.Value());

}

TEST_F(OptionUTest, DefaultTagingWorking){

	options::Option<double>      myDouble("my-double", 1.0);
	options::Option<bool>        myBool(  "my-bool"  , true);
	options::Option<std::string> myString("my-string", "");

	EXPECT_DOUBLE_EQ(1.0,myDouble.Value());
	EXPECT_TRUE(myBool.Value());
	EXPECT_EQ("",myString.Value());

	EXPECT_TRUE(myDouble.IsDefault());
	EXPECT_TRUE(myBool.IsDefault());
	EXPECT_TRUE(myString.IsDefault());


	EXPECT_FALSE(myDouble.Parse(""));
	EXPECT_TRUE(myDouble.IsDefault());

	EXPECT_TRUE(myDouble.Parse("1e6"));
	EXPECT_FALSE(myDouble.IsDefault());

	EXPECT_TRUE(myBool.Parse(""));
	EXPECT_FALSE(myBool.IsDefault());


	EXPECT_TRUE(myString.Parse("foo"));
	EXPECT_FALSE(myString.IsDefault());

}


TEST_F(OptionUTest,OutputIsOk){
	options::Option<double> myDouble("my-double",1.0,"my description");


}
