/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * Env.cpp
 *
 *  Created on: Jan 4, 2013
 *      Author: tuleu
 */

#include "Env.h"

#include <cstdlib>
#include <stdexcept>
#include <errno.h>

#include <sstream>

namespace os {

}

std::string os::Env::Get(const std::string& name , bool throwExcept){
	const char * envVar = getenv(name.c_str());
	if(envVar !=  NULL){
		return envVar;
	} else if(throwExcept){
		throw std::out_of_range("'" + name + "' is not in the environment");
	} else {
		return "";
	}
}

void os::Env::Set(const std::string& name ,
                  const std::string& value ,
                  bool overwrite){
	if(setenv(name.c_str(),value.c_str(),overwrite ? 1 : 0)){
		switch(errno){
		case EINVAL :
			throw std::logic_error("Name '" + name + "' is invalid.");
		case ENOMEM :
			throw std::runtime_error("Unsifficient memory to add variable.");
		default :
			std::ostringstream os;
			os << "Unknown error : " << errno;
			throw std::runtime_error(os.str());
		}
	}
}

void os::Env::Unset(const std::string& name){
	if(unsetenv(name.c_str())){
		if(errno == EINVAL){
			throw std::logic_error("Name '" + name + "' is invalid.");
		} else {
			std::ostringstream os;
			os << "Unknown error : " << errno;
			throw std::runtime_error(os.str());
		}
	}
}

bool os::Env::IsSet(const std::string& name){
	return getenv(name.c_str()) != NULL;
}
 /* namespace os */
