/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * Pathname.h
 *
 *  Created on: 7 nov. 2012
 *      Author: akay
 */

#ifndef BIOROB_CPP_OS_PATHNAME_H_
#define BIOROB_CPP_OS_PATHNAME_H_

#include <string>
#include <stdint.h>


namespace os{

class Pathname {
public:
	Pathname();
	Pathname(const std::string & name);
	~Pathname();


	void SetPathname(const std::string & name);
	const std::string & CompletePath() const;

	bool IsAbsolute() const;
	bool IsRelative() const;


	const std::string & Filename();
	const std::string & Path();
	const std::string & Basename();



	const std::string & Suffix();
	bool HasPrefix(const std::string & prefix);

private :
	enum ComputedFlags{
		NONE                = 0,
		FILENAME_AND_PATH   = 1 << 0,
		BASENAME_AND_SUFFIX = 1 << 1
	};

	void ComputeBaseAndSuffix();
	void ComputeFilenameAndPath();

	uint32_t d_computed;



	std::string d_pathname;
	std::string d_path;
	std::string d_filename;
	std::string d_basename;
	std::string d_suffix;

};

}//namesapce os

#endif /* BIOROB_CPP_OS_PATHNAME_H_ */
